% Simulate improved GPS tracking
% Using Kalman filtering, prediction and smoothing

qv = 2e-3;  % slow, low noise
%qv = 1e-1;   % fast, more noisy
v=4;    % speed m/s
h=1;    % sampling period in seconds
N=50;  % number of samples

% The true trajectory (two straight lines with a turn)
xtrue1=[zeros(1,N+1) 0:v*h:(N-1)*v*h];     
xtrue2=[0:v*h:N*v*h N*v*h*ones(1,N)];
xtrue=[xtrue1;xtrue2];

% The gps measurements
gps_accuracy = 5; % measurement accuracy in meter  
randn('state',0)
ygps = xtrue + gps_accuracy*randn(size(xtrue));
figure(1)
plot(ygps(1,:),ygps(2,:),'linewidth',1.2); 
hold on

% The Kalman filter (matlab notation)
I = eye(2);
A = [I h*I; 0*I I];
G = [0*I;I];
C = [I 0*I];
Q = qv*I;    % tuning parameter
R = gps_accuracy^2*I;
[M,P,Z,E] = dlqe(A,G,C,Q,R);
kalman1 = ss(A-A*M*C,A*M,eye(4),zeros(4,2),h);
kalman0 = ss(A-A*M*C,A*M,eye(4)-M*C,M,h);

% Some plotting
time = (0:2*N)*h;
xpred1 = lsim(kalman1, ygps,time)';
ypred1 = C*xpred1;
plot(ypred1(1,:),ypred1(2,:),'r-','linewidth',1.2)

xpred01 = xpred1 + M*(ygps-C*xpred1);
xpred0 = lsim(kalman0, ygps,time)';
ypred0 = C*xpred0;

%plot(ypred0(1,:),ypred0(2,:),'m-','linewidth',1.2)
plot(xtrue(1,:),xtrue(2,:),'k-')
legend('gps','kalman1','true trajectory','Location','East')
title(['qv=' num2str(qv)],'fontsize',16,'fontweight','b')
% print -depsc gps1.eps


% figure(2)
% plot(xtrue(2,:),'b-x')
% hold on
% plot(xpred1(2,:),'r-x')
% plot(xpred0(2,:),'k-x')
% plot(ygps(2,:),'-mx')

% figure(1)
% a=0.75;
% ydum = filtfilt([1-a 0],[1 -a],ygps');
% plot(ydum(:,1),ydum(:,2),'m-','linewidth',1.2)

 
